// DialogEdView.h : interface of the CDialogEdView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIALOGEDVIEW_H__0824DC2F_826C_11D4_8195_00C04F0F05BA__INCLUDED_)
#define AFX_DIALOGEDVIEW_H__0824DC2F_826C_11D4_8195_00C04F0F05BA__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EditorView.h"
#include "PreviewView.h"
#include "SourceView.h"

#include "HtmlDialog.h"


class CDialogEdView : 
   public CWindowImpl<CDialogEdView, CTabCtrl>,
   public CDynamicChain,
   CTextPersist
{
public:
   DECLARE_WND_SUPERCLASS(NULL, CTabCtrl::GetWndClassName())

   enum { T_EDITOR, T_SOURCE, T_PREVIEW } m_Display;

   CEditorView m_EditorView;
   CPreviewView m_PreviewView;
   CSourceView m_SourceView;
   HWND m_hWndClient;
   CComBSTR m_bstrText;
   CString m_sFilename;

   BEGIN_MSG_MAP(CDialogEdView)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      MESSAGE_HANDLER(WM_SIZE, OnSize)
      COMMAND_ID_HANDLER(ID_FILE_NEW, OnFileNew)
      COMMAND_ID_HANDLER(ID_FILE_OPEN, OnFileLoad)
      COMMAND_ID_HANDLER(ID_FILE_SAVE, OnFileSave)
      COMMAND_ID_HANDLER(ID_FILE_SAVE_AS, OnFileSaveAs)
      COMMAND_ID_HANDLER(ID_VIEW_TESTDIALOG, OnTestDialog)
      COMMAND_ID_HANDLER(ID_FILE_PRINT_SETUP, OnPrintSetup);
      NOTIFY_CODE_HANDLER(TCN_SELCHANGE, OnTabSelect)
      CHAIN_MSG_MAP_DYNAMIC(0)
   END_MSG_MAP()

   // -----------------------------------------

   BOOL PreTranslateMessage(MSG* pMsg)
   {
      switch( m_Display ) {
      case T_EDITOR:
         m_EditorView.PreTranslateMessage(pMsg);
         break;
      case T_PREVIEW:
         m_PreviewView.PreTranslateMessage(pMsg);
         break;
      case T_SOURCE:
         m_SourceView.PreTranslateMessage(pMsg);
         break;
      }
      return FALSE;
   }

   // -----------------------------------------


   LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
   {
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);

      {
         CString s;
         TCITEM tab = { 0 };
         tab.mask = TCIF_TEXT;   
         s.LoadString(IDS_EDIT);
         tab.pszText = (LPTSTR)(LPCTSTR)s;
         InsertItem(0, &tab);
         s.LoadString(IDS_SOURCE);
         tab.pszText = (LPTSTR)(LPCTSTR)s;
         InsertItem(1, &tab);
         s.LoadString(IDS_PREVIEW);
         tab.pszText = (LPTSTR)(LPCTSTR)s;
         InsertItem(2, &tab);

         SetFont( AtlGetStockFont(ANSI_VAR_FONT) );
      }


      __try {
         m_SourceView.Create(m_hWnd, rcDefault);
         m_EditorView.Create(m_hWnd, rcDefault);
         m_PreviewView.Create(m_hWnd, rcDefault);
      }
      __except( EXCEPTION_EXECUTE_HANDLER ) {
         // Oops!
      }

      if( m_SourceView.IsWindow()==FALSE ) {
         ::MessageBox(NULL, _T("The MS RichText Control could not be created.\nMake sure the latest version is correctly installed."), _T("Error"), MB_OK | MB_ICONERROR);
      }
      if( m_EditorView.IsWindow()==FALSE ) {
         ::MessageBox(NULL, _T("The MS DHTML Editing Component could not be created.\nMake sure it is installed."), _T("Error"), MB_OK | MB_ICONERROR);
      }
      

      NewDocument();

      return lRes;
   }

   LRESULT OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& bHandled)
   {
      // Calculate the display rectangle, assuming the 
      // tab control is the size of the client area. 
      RECT rc; 
      ::SetRect(&rc, 0, 0, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam)); 
      AdjustRect(FALSE, &rc); 
      ::MoveWindow(m_hWndClient, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, FALSE);
      bHandled = FALSE;
      return 0;
   }

   LRESULT OnTabSelect(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)
   {
      HWND hwndOld, hwndNew;
      switch( m_Display ) {
      case T_EDITOR:
         hwndOld = m_EditorView;
         break;
      case T_SOURCE:
         hwndOld = m_SourceView;
         break;
      case T_PREVIEW:
         hwndOld = m_PreviewView;
         break;
      default:
         ATLASSERT(false);
         return 0;
      }
      CComBSTR bstrText = GetText();
      switch( GetCurSel() ) {
      case 0:
         m_Display = T_EDITOR;
         hwndNew = m_EditorView;
         SetChainEntry(0, &m_EditorView);
         break;
      case 1:
         m_Display = T_SOURCE;
         hwndNew = m_SourceView;
         SetChainEntry(0, &m_SourceView);
         break;
      case 2:
         m_Display = T_PREVIEW;
         hwndNew = m_PreviewView;
         SetChainEntry(0, &m_PreviewView);
         break;
      default:
         ATLASSERT(false);
         return 0;
      }
      SetText(bstrText);
      m_hWndClient = hwndNew;
      // Fake resize
      RECT rc;
      GetWindowRect(&rc);
      BOOL bDummy;
      OnSize(WM_SIZE, MAKEWPARAM(rc.left,rc.top), MAKELPARAM(rc.right-rc.left, rc.bottom-rc.top), bDummy);
      //
      ::ShowWindow(hwndNew, SW_SHOW);
      ::ShowWindow(hwndOld, SW_HIDE);
      return 0;
   };

   // -----------------------------------------

   LRESULT OnFileNew(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      NewDocument();
      return 0;
   }

   LRESULT OnFileLoad(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CFileDialog dlg(TRUE);
      if( dlg.DoModal()==IDOK ) {
         if( LoadDocument(dlg.m_ofn.lpstrFile)==false ) {
            CString s;
            s.LoadString(IDS_ERR_FILELOAD);
            MessageBox(s,NULL,MB_OK|MB_ICONERROR);
         };
      }
      return 0;
   }

   LRESULT OnFileSave(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      if( m_sFilename.GetLength()==0 ) {
         CFileDialog dlg(FALSE);
         dlg.DoModal();
         m_sFilename = dlg.m_ofn.lpstrFile;
      }
      if( m_sFilename.GetLength()>0 ) {
         if( SaveDocument(m_sFilename)==false ) {
            CString s;
            s.LoadString(IDS_ERR_SAVEFILE);
            MessageBox(s,NULL,MB_OK|MB_ICONERROR);
         }
      }
      return 0;
   }

   LRESULT OnFileSaveAs(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CFileDialog dlg(FALSE);
      if( dlg.DoModal()==IDOK) {
         m_sFilename = dlg.m_ofn.lpstrFile;
         if( SaveDocument(m_sFilename)==false ) {
            CString s;
            s.LoadString(IDS_ERR_SAVEFILE);
            MessageBox(s,NULL,MB_OK|MB_ICONERROR);
         }
      }
      return 0;
   }

   LRESULT OnTestDialog(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CHtmlDialog dlg;
      CComVariant v, vOut;
      CComBSTR sOptions = L"";
      dlg.DoModal(m_hWnd, GetText(), &v, sOptions, &vOut);
      return 0;
   }

   LRESULT OnPrintSetup(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CPrintDialog dlg(TRUE);
      dlg.DoModal();
      return 0;
   }

   // -----------------------------------------

   bool NewDocument()
   {
      m_bstrText = AtlLoadHTML(IDR_HTML_NEW);
      m_Display = T_EDITOR;
      m_hWndClient = m_EditorView;
      m_EditorView.SetText(m_bstrText);
      SetChainEntry(0, &m_EditorView);
      m_EditorView.ShowWindow(SW_SHOW);
      SetCurSel(0); // Make sure we are on the Editor page
      return true;
   }

   bool LoadDocument(LPCTSTR pstrFilename)
   {
      ATLASSERT(pstrFilename);
      NewDocument();
      CFile f;
      if( f.Open(pstrFilename)==FALSE ) return false;
      DWORD dwSize = f.GetSize();
      CString s;
      LPTSTR pStr = s.GetBuffer(dwSize+1);
      f.Read(pStr, dwSize);
      pStr[dwSize] = _T('\0');
      f.Close();
      s.ReleaseBuffer();
      CComBSTR bstr = s;
      SetText(bstr);
      return true;
   }

   bool SaveDocument(LPCTSTR pstrFilename)
   {
      ATLASSERT(pstrFilename);
      CComBSTR bstrText = GetText();
      CString sText(bstrText);
      CFile f;
      if( f.Create(pstrFilename)==FALSE ) return false;
      f.Write(sText, ::lstrlen(sText));
      f.Close();
      m_sFilename = pstrFilename;
      return true;
   }

   // -----------------------------------

   CComBSTR GetText()
   {
      CComBSTR bstrText;
      switch( m_Display ) {
      case T_EDITOR:
         bstrText = m_EditorView.GetText();
         break;
      case T_SOURCE:
         bstrText = m_SourceView.GetText();
         break;
      case T_PREVIEW:
         bstrText = m_PreviewView.GetText();
         break;
      }
      return bstrText;
   };

   void SetText(CComBSTR &bstrText)
   {
      switch( m_Display ) {
      case T_EDITOR:
         m_EditorView.SetText(bstrText);
         break;
      case T_SOURCE:
         m_SourceView.SetText(bstrText);
         break;
      case T_PREVIEW:
         m_PreviewView.SetText(bstrText);
         break;
      }
   }

};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DIALOGEDVIEW_H__0824DC2F_826C_11D4_8195_00C04F0F05BA__INCLUDED_)
