#if !defined(AFX_EDITORVIEW_H__20000904_A4D7_DBE3_2E5C_0080AD509054__INCLUDED_)
#define AFX_EDITORVIEW_H__20000904_A4D7_DBE3_2E5C_0080AD509054__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "EventsDlg.h"
#include "PropertiesDlg.h"
#include "DataObject.h"
#include "DropSource.h"

#define ID_CTL 0

#define ID_TOOL_START ID_TOOL_ARROW
#define ID_TOOL_END ID_TOOL_LIST


class CEditorView : 
   public CWindowImpl<CEditorView, CAxWindow>,
   public CTextPersist,
   public IDispEventImpl<ID_CTL, CEditorView>
{
public:
   DECLARE_WND_SUPERCLASS(NULL, CAxWindow::GetWndClassName())

   CComQIPtr<IDHTMLEdit> m_spDHTML;
   int m_iUniqueId;

   BEGIN_MSG_MAP(CEditorView)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
      COMMAND_ID_HANDLER(ID_FILE_PRINT, OnPrint)
      COMMAND_ID_HANDLER(ID_EDIT_COPY, OnEditCopy)
      COMMAND_ID_HANDLER(ID_EDIT_CUT, OnEditCut)
      COMMAND_ID_HANDLER(ID_EDIT_PASTE, OnEditPaste)
      COMMAND_ID_HANDLER(ID_EDIT_DELETE, OnEditDelete)
      COMMAND_ID_HANDLER(ID_EDIT_UNDO, OnUndo)
      COMMAND_ID_HANDLER(ID_EDIT_REDO, OnRedo)
      COMMAND_ID_HANDLER(ID_POPUP_SENDFRONT, OnSendToFront)
      COMMAND_ID_HANDLER(ID_POPUP_SENDBACK, OnSendToBack)
      COMMAND_ID_HANDLER(ID_POPUP_EVENTS, OnShowEvents)
      COMMAND_ID_HANDLER(ID_POPUP_PROPERTIES, OnShowProperties)
      COMMAND_ID_HANDLER(ID_POPUP_TESTDIALOG, OnTestDialog)
      NOTIFY_CODE_HANDLER(TBN_DRAGOUT, OnToolDrag)
   END_MSG_MAP()

   BEGIN_SINK_MAP(CEditorView)
      // Sink events from the default ActiveX interface
      SINK_ENTRY(ID_CTL, 0x00000002, OnDisplayChanged)
      SINK_ENTRY(ID_CTL, 0x00000003, OnShowContextMenu)
   END_SINK_MAP()

   // -----------------------------------

   HWND Create(HWND hWndParent, RECT& rcPos)
   {
      DWORD dwFlags = WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
      return CWindowImpl<CEditorView, CAxWindow>::Create(hWndParent, rcPos, _T("DHTMLEdit.DHTMLEdit"), dwFlags);
   }

   // In order for AtlAdviseSinkMap() to work, we need this overload!?
   HWND GetDlgItem(int /*nID*/) const { return m_hWnd; };

   // --------------------------------------
   
 
   BOOL PreTranslateMessage(MSG* pMsg)
   {
      if((pMsg->message < WM_KEYFIRST || pMsg->message > WM_KEYLAST) &&
         (pMsg->message < WM_MOUSEFIRST || pMsg->message > WM_MOUSELAST))
         return FALSE;

      // Give OCX a chance to translate this message
      return (BOOL)SendMessage(WM_FORWARDMSG, 0, (LPARAM)pMsg);
   }

   // --------------------------------------

   LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
   {
      // Create the control for me
      DefWindowProc(uMsg, wParam, lParam);

      HRESULT hr;
      hr = QueryControl(IID_IDHTMLEdit, (LPVOID *)&m_spDHTML);
      if( FAILED(hr) ) return 1;
      m_spDHTML->put_AbsoluteDropMode(VARIANT_TRUE);
      m_spDHTML->put_SourceCodePreservation(VARIANT_TRUE);
      m_spDHTML->put_UseDivOnCarriageReturn(VARIANT_FALSE);

      // Establish connection points
      AtlAdviseSinkMap(this, true); 
      m_iUniqueId = 1;
 
      bHandled = TRUE;
      return 0;
   }

   LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      // Controls still live? Tear down connection points
      //AtlAdviseSinkMap(this, false); 
      return 0;
   }

   LRESULT OnToolDrag(int /*idCtrl*/, LPNMHDR pnmh, BOOL& /*bHandled*/)
   {
      LPNMTOOLBAR lpnmtb = (LPNMTOOLBAR)pnmh;
      UINT id = lpnmtb->iItem;
      if( id<ID_TOOL_START || id>ID_TOOL_END ) return 0;
      if( id==ID_TOOL_ARROW ) return 0;

      VARIANT_BOOL vb;
      m_spDHTML->get_AbsoluteDropMode(&vb);

      // Create drag'n'drop objects
      HRESULT Hr;
      CComObject<CDataObject> *pDataObject;
      CComObject<CDropSource> *pDropSource;
      Hr = CComObject<CDropSource>::CreateInstance(&pDropSource);
      if( FAILED(Hr) ) return 0;
      Hr = CComObject<CDataObject>::CreateInstance(&pDataObject);
      if( FAILED(Hr) ) return 0;
      pDataObject->m_sText = GetToolElement(id);
      // Let system handle drag operation...
      DWORD dwEffect;
      Hr = ::DoDragDrop(pDataObject, pDropSource, DROPEFFECT_COPY, &dwEffect);
      if( FAILED(Hr) ) return 0;

      return 0;
   }

   // -----------------------------------------

   CComBSTR GetText()
   {
      ATLASSERT(m_spDHTML);
      WaitBusy();
      CComBSTR bstr;
      HRESULT hr = m_spDHTML->get_DocumentHTML(&bstr);
      if( FAILED(hr) ) return NULL;
      return bstr;
   }
   void SetText(CComBSTR &bstr)
   {
      ATLASSERT(m_spDHTML);
      HRESULT Hr;
      Hr = m_spDHTML->put_DocumentHTML(bstr);
      if( FAILED(Hr) ) return;
      WaitBusy();
   }

   // --------------------------------------

   void __stdcall OnDisplayChanged()
   {
   }

   void __stdcall OnShowContextMenu(int x, int y)
   {
      CComPtr<IHTMLSelectionObject> spSel;
      HRESULT hr;
      hr = GetSelectedElement(spSel);
      if( FAILED(hr) ) return;
      CComBSTR bstrType;
      hr = spSel->get_type(&bstrType);
      if( FAILED(hr) ) return;
      CMenu menuContext;
      if( bstrType==L"None" ) {
         menuContext.LoadMenu(IDR_POPUP_DIALOG);
      }
      else if( bstrType==L"Control" ) {
         menuContext.LoadMenu(IDR_POPUP_CONTROL);
      }
      else {
         return; // no element we care about...
      }
      CPoint pt(x,y);
      ClientToScreen(&pt);
      CMenuHandle menuPopup(menuContext.GetSubMenu(0));
      ::SetMenuDefaultItem(menuPopup, ID_POPUP_PROPERTIES, FALSE);
      ::SetForegroundWindow(m_hWnd);
      menuPopup.TrackPopupMenu( 
         TPM_LEFTALIGN | TPM_RIGHTBUTTON, 
         pt.x, pt.y, 
         m_hWnd, 
         NULL);
   }

   // --------------------------------

   LRESULT OnSendToFront(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      WaitBusy();
      CComVariant vIn, vOut;
      m_spDHTML->ExecCommand(DECMD_BRING_TO_FRONT, OLECMDEXECOPT_DONTPROMPTUSER, &vIn, &vOut);
      return 0;
   }

   LRESULT OnSendToBack(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CComVariant vIn, vOut;
      m_spDHTML->ExecCommand(DECMD_SEND_TO_BACK, OLECMDEXECOPT_DONTPROMPTUSER, &vIn, &vOut);
      return 0;
   }

   LRESULT OnEditCopy(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CComVariant vIn, vOut;
      m_spDHTML->ExecCommand(DECMD_COPY, OLECMDEXECOPT_DONTPROMPTUSER, &vIn, &vOut);
      return 0;
   }

   LRESULT OnEditCut(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CComVariant vIn, vOut;
      m_spDHTML->ExecCommand(DECMD_CUT, OLECMDEXECOPT_DONTPROMPTUSER, &vIn, &vOut);
      return 0;
   }

   LRESULT OnEditPaste(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CComVariant vIn, vOut;
      m_spDHTML->ExecCommand(DECMD_PASTE, OLECMDEXECOPT_DONTPROMPTUSER, &vIn, &vOut);
      return 0;
   }

   LRESULT OnEditDelete(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CComVariant vIn, vOut;
      m_spDHTML->ExecCommand(DECMD_DELETE, OLECMDEXECOPT_DONTPROMPTUSER, &vIn, &vOut);
      return 0;
   }

   LRESULT OnUndo(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CComVariant vIn, vOut;
      m_spDHTML->ExecCommand(DECMD_UNDO, OLECMDEXECOPT_DONTPROMPTUSER, &vIn, &vOut);
      return 0;
   }

   LRESULT OnRedo(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CComVariant vIn, vOut;
      m_spDHTML->ExecCommand(DECMD_REDO, OLECMDEXECOPT_DONTPROMPTUSER, &vIn, &vOut);
      return 0;
   }

   LRESULT OnPrint(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CComVariant vIn;
      m_spDHTML->PrintDocument(&vIn);
      return 0;
   }

   LRESULT OnShowProperties(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CComBSTR bstrType;
      CComPtr<IHTMLElement> spEl;
      if( FAILED( GetSelectedElement(bstrType, spEl) ) ) return 1;
      CPropertiesDlg dlg(m_spDHTML, bstrType, spEl);
      if( dlg.DoModal()==IDOK ) {
         // Maybe we should reparse document?
         //SendMessage(GetParent(), WM_NOTIFY, ...);
      };
      return 0;
   }

   LRESULT OnTestDialog(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      SendMessage(GetParent(), WM_COMMAND, ID_VIEW_TESTDIALOG, 0);
      return 0;
   }

   LRESULT OnShowEvents(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CComBSTR bstrType;
      CComPtr<IHTMLElement> spEl;
      if( FAILED( GetSelectedElement(bstrType, spEl) ) ) return 1;
      CEventsDlg dlg(m_spDHTML, bstrType, spEl);
      if( dlg.DoModal()==IDOK ) {
         // Script code was added to the document, so jump
         // to source-view now!
         //SendMessage(GetParent(), WM_NOTIFY, ...);
      };
      return 0;
   }

   // --------------------------------

private:
   CString GetToolElement(UINT id)
   {
      TCHAR szBuffer[256];
      switch( id ) {
      case ID_TOOL_ARROW:
         return "";
      case ID_TOOL_LABEL:
         ::wsprintf(szBuffer, _T("<span id=\"LABEL%d\" style=\"position: absolute\">Label</span>"), m_iUniqueId++);
         return szBuffer;
      case ID_TOOL_EDIT:
         ::wsprintf(szBuffer, _T("<textarea id=\"TEXT%d\">Text</textarea>"), m_iUniqueId++);
         return szBuffer;
      case ID_TOOL_BUTTON:
         ::wsprintf(szBuffer, _T("<button id=\"BUTTON%d\">Button</button>"), m_iUniqueId++);
         return szBuffer;
      case ID_TOOL_CHOICE:
         ::wsprintf(szBuffer, _T("<select ID=\"CHOICE%d\" SIZE=1><option value=\"1\"></select>"), m_iUniqueId++);
         return szBuffer;
      case ID_TOOL_CHECKBOX:
         ::wsprintf(szBuffer, _T("<input id=\"CHECKBOX%d\" type=\"checkbox\">"), m_iUniqueId++);
         return szBuffer;
      case ID_TOOL_OPTION:
         ::wsprintf(szBuffer, _T("<input id=\"OPTION%d\" type=\"RADIO\">"), m_iUniqueId++);
         return szBuffer;
      case ID_TOOL_LIST:
         ::wsprintf(szBuffer, _T("<select id=\"LIST%d\" size=5><option value=\"1\"></select>"), m_iUniqueId++);
         return szBuffer;
      case ID_TOOL_FRAME:
         ::wsprintf(szBuffer, _T("<hr id=\"FRAME%d\" style=\"WIDTH: 50px; HEIGHT: 50px\">"), m_iUniqueId++);
         return szBuffer;
      case ID_TOOL_IMAGE:
         ::wsprintf(szBuffer, _T("<img id=\"IMAGE%d\" src=\"\">"), m_iUniqueId++);
         return szBuffer;
      default:
         ATLASSERT(NULL=="Invalid tool id");
         return "";
      }
   }

   void WaitBusy() const
   {
      ATLASSERT(m_spDHTML);
      // Wait until document has been read.
      //VARIANT_BOOL vbBusy;
      //while( SUCCEEDED(m_spDHTML->get_Busy(&vbBusy)) && vbBusy==VARIANT_TRUE ) ::Sleep(0);
   }

   HRESULT GetSelectedElement(CComPtr<IHTMLSelectionObject> &spSel )
   {
      CComPtr<IHTMLDocument2> spDOM;
      HRESULT hr;
      hr = m_spDHTML->get_DOM(&spDOM);
      if( FAILED(hr) ) return hr;
      hr = spDOM->get_selection(&spSel);
      return hr;
   }

   HRESULT GetSelectedElement(CComBSTR &bstrType, CComPtr<IHTMLElement> &spEl)
   {
      CComPtr<IHTMLSelectionObject> spSel;
      HRESULT hr;
      hr = GetSelectedElement(spSel);
      if( FAILED(hr) ) return hr;
      hr = spSel->get_type(&bstrType);
      if( FAILED(hr) ) return hr;
      if( bstrType==L"None" || bstrType==L"Text" ) {
         // This reduces the selection to just the insertion
         // point. The parentElement method will then return the
         // element directly under the mouse pointer.
         CComPtr<IDispatch> spDisp;
         hr = spSel->createRange(&spDisp);
         CComQIPtr<IHTMLTxtRange> spText( spDisp );
         if( spText==NULL ) return E_NOINTERFACE;
         spText->collapse(VARIANT_TRUE);
         hr = spText->parentElement(&spEl);
      }
      else if( bstrType==L"Control" ) {
         // A control is selected. The commonParentElement
         // will return the site selected element.
         CComPtr<IDispatch> spDisp;
         hr = spSel->createRange(&spDisp);
         if( FAILED(hr) ) return hr;
         CComQIPtr<IHTMLControlRange> spRange( spDisp );
         if( spRange==NULL ) return E_NOINTERFACE;
         hr = spRange->commonParentElement(&spEl);
      }
      else
         hr = E_FAIL;
      return hr;
   }
};


#endif // !defined(AFX_EDITORVIEW_H__20000904_A4D7_DBE3_2E5C_0080AD509054__INCLUDED_)

