#if !defined(AFX_EVENTSDLG_H__20000930_3684_D6DF_2D68_0080AD509054__INCLUDED_)
#define AFX_EVENTSDLG_H__20000930_3684_D6DF_2D68_0080AD509054__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CEventsDlg : public CDialogImpl<CEventsDlg>
{
public:
   enum { IDD = IDD_EVENTS };

   CTreeViewCtrlEx m_ctlList;
   CComBSTR m_bstrType;
   CComPtr<IDHTMLEdit> m_spDHTML;
   CComPtr<IHTMLElement> m_spEl;
   CComDispatchDriver m_disp;
   CComPtr<IHTMLElementCollection> m_spScripts;
   CComBSTR m_bstrID;

   BEGIN_MSG_MAP(CEventsDlg)
      MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
      COMMAND_ID_HANDLER(IDOK, OnOk)
      COMMAND_ID_HANDLER(IDCANCEL, OnCancel)
      NOTIFY_CODE_HANDLER(NM_DBLCLK, OnListDblClick)
   END_MSG_MAP()

   CEventsDlg(IDHTMLEdit *pDHTML, BSTR bstrType, IHTMLElement *pElement)
   {
      ATLASSERT(pDHTML);
      ATLASSERT(bstrType);
      ATLASSERT(pElement);
      m_spDHTML = pDHTML;
      m_bstrType = bstrType;
      m_spEl = pElement;
   };

   LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      ATLASSERT(m_spEl);

      m_disp = m_spEl;
      //
      HRESULT hr;
      CComPtr<IHTMLDocument2> spDOM;
      hr = m_spDHTML->get_DOM(&spDOM);
      if( FAILED(hr) ) return 1;
      hr = spDOM->get_scripts(&m_spScripts);
      if( FAILED(hr) ) return 1;
      //
      hr = m_spEl->get_id(&m_bstrID);
      if( FAILED(hr) ) return 1;
      if( m_bstrID.Length()==0 ) return 1;

      // Attach to the display list.
      // It's actually a TreeView because we need to bold item text.
      m_ctlList.Attach(GetDlgItem(IDC_LIST));
      InitEventList();
      HighlightUsedEvents();
      m_ctlList.SortChildren(m_ctlList.GetNextItem(0,TVGN_ROOT));

      CenterWindow(GetParent());
      return TRUE;
   }

   LRESULT OnListDblClick(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)
   {
      USES_CONVERSION;

      CTreeItem itm(m_ctlList.GetSelectedItem(), &m_ctlList);
      if( itm.IsNull() ) return 0;
      
      // Check that the script isn't already defined
      CString sEventName;
      itm.GetText(sEventName);
      if( HasScriptBlock(sEventName)==S_OK ) {
         EndDialog(IDOK);
         return 0;
      }

      TCHAR szScript[512];
      ::wsprintf(szScript, _T("\n%s<SCRIPT DEFER FOR=\"%ls\" EVENT=\"%s()\" LANGUAGE=\"VBScript\">\nMsgBox \"Event\" \n</SCRIPT>"),
         _T("&nbsp;"), // this fixes an IE bug where no script would get added
         m_bstrID,
         (LPCTSTR)sEventName);

      // Add script code to end of BODY
      HRESULT hr;
      CComPtr<IHTMLDocument2> spDOM;
      hr = m_spDHTML->get_DOM(&spDOM);
      if( FAILED(hr) ) return hr;
      CComPtr<IHTMLElement> spBody;
      hr = spDOM->get_body(&spBody);
      if( FAILED(hr) ) return hr;
      hr = spBody->insertAdjacentHTML(CComBSTR(L"BeforeEnd"), CComBSTR(szScript));
      if( FAILED(hr) ) return hr;

      // Close the dialog upon success...
      EndDialog(IDOK);
      return 0;
   }

   LRESULT OnOk(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      EndDialog(IDCANCEL);
      return 0;
   }

   LRESULT OnCancel(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      EndDialog(IDCANCEL);
      return 0;
   }

   // ------------------------

   HRESULT HasScriptBlock(LPCTSTR pstrEventName)
   {
      ATLASSERT(pstrEventName);
      ATLASSERT(m_spScripts);
      ATLASSERT(m_disp.p);
      USES_CONVERSION;
      HRESULT hr;

      CComBSTR bstrEventName(pstrEventName);
      bstrEventName += L"()";

      // Scan all the SCRIPT elements
      long dwCnt;
      m_spScripts->get_length(&dwCnt);
      CComVariant vEmpty;
      for( long i=0; i<dwCnt; i++ ) {
         CComPtr<IDispatch> spDisp;
         m_spScripts->item(CComVariant(i), vEmpty, &spDisp);
         CComQIPtr<IHTMLScriptElement> spEl(spDisp);
         if( spEl!=NULL ) {
            CComBSTR bstrFor;
            CComBSTR bstrEvent;
            spEl->get_htmlFor(&bstrFor);
            spEl->get_event(&bstrEvent);
            if( bstrFor!=NULL && bstrEvent!=NULL ) {
               if( (wcsicmp(bstrFor, m_bstrID)==0) &&
                   (wcsicmp(bstrEvent, bstrEventName)==0) )
                     return S_OK; // found it
            }
         }         
      }


      // Might be attached directly to the function. Check
      // intrinsic property.
      DISPID dispid;
      LPWSTR pwstrName = const_cast<LPWSTR>(T2CW(pstrEventName));
      hr = m_disp.p->GetIDsOfNames(IID_NULL, 
         &pwstrName, 
         1, 
         LOCALE_SYSTEM_DEFAULT, 
         &dispid);
      if( FAILED(hr) ) return E_INVALIDARG;
      CComVariant v;
      hr = m_disp.GetProperty(dispid, &v);
      if( FAILED(hr) ) return S_FALSE;
      if( v.vt!=VT_DISPATCH ) return S_FALSE;
      return S_OK;
   }

   inline void AddItem(LPCTSTR pstrText)
   {
      ATLASSERT(pstrText);
      m_ctlList.InsertItem(pstrText, TVI_ROOT, TVI_LAST);
   }

   void InitEventList()
   {
      CComBSTR bstrTag;
      m_spEl->get_tagName(&bstrTag);
      if( m_bstrType==L"Control" ) {
         // Control properties
         AddItem(_T("onclick"));
         AddItem(_T("onresize"));
         AddItem(_T("onblur"));
         AddItem(_T("onfocus"));
         if( !(bstrTag==L"HR") ) {
            AddItem(_T("onmousemove"));
            AddItem(_T("onmouseover"));
            AddItem(_T("onmouseout"));
            AddItem(_T("onmousedown"));
            AddItem(_T("onmouseup"));
         }
         if( bstrTag==L"TEXTAREA" ) {
            AddItem(_T("onkeyup"));
            AddItem(_T("onkeydown"));
            AddItem(_T("onkeypress"));
            AddItem(_T("onselect"));
            AddItem(_T("onscroll"));
         }
         if( bstrTag==L"SELECT" ) {
            AddItem(_T("onchange"));
            AddItem(_T("onscroll"));
         }
      }
      else {
         // Dialog properties
         AddItem(_T("onload"));
         AddItem(_T("onunload"));
         AddItem(_T("onclick"));
         AddItem(_T("onresize"));
         AddItem(_T("onblur"));
         AddItem(_T("onfocus"));
      }
   }

   void HighlightUsedEvents()
   {
      CTreeItem itm(m_ctlList.GetNextItem(0, TVGN_ROOT), &m_ctlList);
      while( itm.IsNull()==FALSE ) {
         CString sEventName;
         itm.GetText(sEventName);
         if( HasScriptBlock(sEventName)==S_OK ) {
            m_ctlList.SetItemState(itm, TVIS_BOLD, (UINT)-1);
         }
         itm = itm.GetNextSibling();
      }
   }
};


#endif // !defined(AFX_EVENTSDLG_H__20000930_3684_D6DF_2D68_0080AD509054__INCLUDED_)

