#if !defined(AFX_PROPERTIESDLG_H__20000930_BA61_4089_209E_0080AD509054__INCLUDED_)
#define AFX_PROPERTIESDLG_H__20000930_BA61_4089_209E_0080AD509054__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "PropertyView.h"

typedef enum { PROPHTML_NONE, PROPHTML_TAG, PROPHTML_ATTRIB, PROPHTML_STYLE } PROPERTY_HTMLTYPE;

class CHtmlProperty : public CProperty
{
public:
   CHtmlProperty( PROPERTY_ITEMTYPE Type, 
      LPCTSTR pstrTitle, 
      VARIANT *pvValue, 
      BOOL bReadOnly=FALSE,
      PROPERTY_HTMLTYPE HtmlType=PROPHTML_NONE,
      LPCTSTR pstrHtmlProperty=NULL)
      : CProperty(Type,pstrTitle,pvValue,bReadOnly)
   {
      m_HtmlType = HtmlType;
      m_sHtmlProperty = pstrHtmlProperty;
   }

   PROPERTY_HTMLTYPE m_HtmlType;
   CString m_sHtmlProperty;
};


class CPropertiesDlg : public CDialogImpl<CPropertiesDlg>
{
public:
   enum { IDD = IDD_PROPERTIES };

   CPropertyViewCtrl m_ctlList;
   CComBSTR m_bstrType;
   CComPtr<IDHTMLEdit> m_spDHTML;
   CComPtr<IHTMLElement> m_spEl;
   CComPtr<IHTMLStyle> m_spStyle;

   BEGIN_MSG_MAP(CPropertiesDlg)
      MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
      COMMAND_ID_HANDLER(IDOK, OnOk)
      COMMAND_ID_HANDLER(IDCANCEL, OnCancel)
      REFLECT_NOTIFICATIONS()
   END_MSG_MAP()

   CPropertiesDlg(IDHTMLEdit *pDHTML, BSTR bstrType, IHTMLElement *pElement)
   {
      ATLASSERT(pDHTML);
      ATLASSERT(bstrType);
      ATLASSERT(pElement);
      m_spDHTML = pDHTML;
      m_bstrType = bstrType;
      m_spEl = pElement;
      m_spEl->get_style(&m_spStyle);
   };

   LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      m_ctlList.SubclassWindow(GetDlgItem(IDC_LIST));

      CComBSTR bstr;
      CComVariant v;
      CComVariant vEmpty;
      CString s;

      AddGroupItem(IDS_PROP_GENERAL);
      AddAttribItem(IDS_PROP_ID, _T("ID"));
      //
      AddGroupItem(IDS_PROP_POSITION);
      AddStyleItem(IDS_PROP_LEFT, _T("LEFT"));
      AddStyleItem(IDS_PROP_TOP, _T("TOP"));
      AddStyleItem(IDS_PROP_WIDTH, _T("WIDTH"));
      AddStyleItem(IDS_PROP_HEIGHT, _T("HEIGHT"));

      CComBSTR bstrTag;
      m_spEl->get_tagName(&bstrTag);

      if( bstrTag==L"TEXTAREA" ) {
         AddGroupItem(IDS_PROP_MISC);
         AddAttribItem(IDS_PROP_COLS, _T("COLS"));
         AddAttribItem(IDS_PROP_ROWS, _T("ROWS"));
         AddAttribItem(IDS_PROP_READONLY, _T("READONLY"));
         AddAttribItem(IDS_PROP_TOOLTIP, _T("TITLE"));
         AddAttribItem(IDS_PROP_WRAP, _T("WRAP"));
      }
      else if( bstrTag==L"BUTTON" ) {
         AddGroupItem(IDS_PROP_MISC);
         AddAttribItem(IDS_PROP_VALUE, _T("VALUE"));
         AddAttribItem(IDS_PROP_DISABLED, _T("DISABLED"));
      }
      else if( bstrTag==L"OPTION" ) {
         AddGroupItem(IDS_PROP_MISC);
         AddAttribItem(IDS_PROP_SELECTED, _T("SELECTED"));
         AddAttribItem(IDS_PROP_VALUE, _T("VALUE"));
      }
      else if( bstrTag==L"HR" ) {
         AddGroupItem(IDS_PROP_MISC);
         AddAttribItem(IDS_PROP_TOOLTIP, _T("TITLE"));
         AddTagItem(IDS_PROP_NOSHADE, _T("NOSHADE"));
      }
      else if( bstrTag==L"SELECT" ) {
         AddGroupItem(IDS_PROP_MISC);
         AddAttribItem(IDS_PROP_SIZE, _T("SIZE"));
         AddAttribItem(IDS_PROP_DISABLED, _T("DISABLED"));
         AddTagItem(IDS_PROP_MULTISELECT, _T("MULTI"));
      }
      else if( bstrTag==L"IMG" ) {
         AddGroupItem(IDS_PROP_MISC);
         AddAttribItem(IDS_PROP_IMAGE, _T("IMAGE"));
         AddAttribItem(IDS_PROP_TOOLTIP, _T("TITLE"));
         AddAttribItem(IDS_PROP_BORDER, _T("BORDER"));
         AddAttribItem(IDS_PROP_HSPACE, _T("HSPACE"));
         AddAttribItem(IDS_PROP_VSPACE, _T("VSPACE"));
      }

      CenterWindow(GetParent());
      return TRUE;
   }

   LRESULT OnOk(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      EndDialog(wID);
      return 0;
   }

   LRESULT OnCancel(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      EndDialog(wID);
      return 0;
   }

   // ------------------------------

   void AddGroupItem(UINT idString)
   {
      CString s;
      s.LoadString(idString);
      CComVariant vEmpty;
      m_ctlList.AddItem(new CHtmlProperty(PROP_GROUP, s, &vEmpty));
   }

   void AddTagItem(UINT idString, LPCTSTR TagName)
   {
      ATLASSERT(m_spEl);
      CString s;
      s.LoadString(idString);
      CComVariant v;
      m_spEl->getAttribute(CComBSTR(TagName), 0, &v);
      m_ctlList.AddItem(new CHtmlProperty(PROP_ITEM, s, &v, FALSE, PROPHTML_TAG, TagName));
   }

   void AddAttribItem(UINT idString, LPCTSTR TagName)
   {
      ATLASSERT(m_spEl);
      CString s;
      s.LoadString(idString);
      CComVariant v;
      m_spEl->getAttribute(CComBSTR(TagName), 0, &v);
      m_ctlList.AddItem(new CHtmlProperty(PROP_ITEM, s, &v, FALSE, PROPHTML_ATTRIB, TagName));
   }

   void AddStyleItem(UINT idString, LPCTSTR TagName)
   {
      if( m_spStyle==NULL ) return; // no style?
      CString s;
      s.LoadString(idString);
      CComVariant v;
      m_spStyle->getAttribute(CComBSTR(TagName), 0, &v);
      m_ctlList.AddItem(new CHtmlProperty(PROP_ITEM, s, &v, FALSE, PROPHTML_TAG, TagName));
   }

};


#endif // !defined(AFX_PROPERTIESDLG_H__20000930_BA61_4089_209E_0080AD509054__INCLUDED_)

