// RichView.h : interface of the CSourceView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_SOURCEVIEW_H__5574832C_580A_4772_A047_5B5233984B67__INCLUDED_)
#define AFX_SOURCEVIEW_H__5574832C_580A_4772_A047_5B5233984B67__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "HtmlFormat.h"

class CSourceView : 
   public CWindowImpl<CSourceView, CRichEditCtrl>,
   public CTextPersist
{
private:
   typedef struct tagSTREAMINFO {
      CString s;
      int pos;
      int length;
   } STREAMINFO;
   bool m_bIsLoading;

public:
   DECLARE_WND_SUPERCLASS(NULL, CRichEditCtrl::GetWndClassName())

   BEGIN_MSG_MAP(CSourceView)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      NOTIFY_CODE_HANDLER(EN_PROTECTED, OnProtected)
      COMMAND_ID_HANDLER(ID_EDIT_UNDO, OnEditUndo)
#if (_RICHEDIT_VER >= 0x0200)
      COMMAND_ID_HANDLER(ID_EDIT_REDO, OnEditRedo)
#endif
      COMMAND_ID_HANDLER(ID_EDIT_CUT, OnEditCut)
      COMMAND_ID_HANDLER(ID_EDIT_COPY, OnEditCopy)
      COMMAND_ID_HANDLER(ID_EDIT_PASTE, OnEditPaste)
      COMMAND_ID_HANDLER(ID_EDIT_CLEAR, OnEditClear)
   END_MSG_MAP()

   // -----------------------------------

   HWND Create(HWND hWndParent, RECT& rcDefault)
   {
      ATLASSERT(::IsWindow(hWndParent));
      DWORD dwFlags = WS_CHILD | 
                      WS_CLIPSIBLINGS | 
                      WS_CLIPCHILDREN | 
                      WS_BORDER | 
                      WS_HSCROLL | 
                      WS_VSCROLL | 
                      ES_AUTOHSCROLL | 
                      ES_AUTOVSCROLL | 
                      ES_MULTILINE | 
                      ES_NOHIDESEL | 
                      ES_SAVESEL;
      return CWindowImpl<CSourceView, CRichEditCtrl>::Create(hWndParent, rcDefault, NULL, dwFlags);
   }

   // -----------------------------------
   
   LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      // Create the control for me
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);

      m_bIsLoading = false;
      LimitText(400000);
      SetEventMask(ENM_PROTECTED);
      SetModify(FALSE);
      
      return lRes;
   }

   BOOL PreTranslateMessage(MSG* pMsg)
   {
      pMsg;
      return FALSE;
   }

   // -----------------------------------

   LRESULT OnProtected(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)
   {
      //ENPROTECTED *lpnmtb = (ENPROTECTED *)pnmh;
      return m_bIsLoading ? 0 : 1; // prevent modify when not loading new text
   }

   // -----------------------------------

   LRESULT OnEditUndo(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      Undo();
      return 0;
   }

   LRESULT OnEditRedo(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      Redo();
      return 0;
   }

   LRESULT OnEditCut(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      Cut();
      return 0;
   }

   LRESULT OnEditCopy(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      Copy();
      return 0;
   }

   LRESULT OnEditPaste(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      PasteSpecial(CF_TEXT);
      return 0;
   }

   LRESULT OnEditClear(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      Clear();
      return 0;
   }

   // -----------------------------------

   static DWORD CALLBACK MyStreamOutCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
   {
      STREAMINFO *pInfo = (STREAMINFO *)dwCookie;
      // Need null terminated string before we can add it...
      LPTSTR p = new TCHAR[cb+1];
      if( p==NULL ) return 0;
      ::CopyMemory(p, pbBuff, cb);
      p[cb] = _T('\0');
      pInfo->s += p;
      delete [] p;
      pInfo->length += cb;
      *pcb = cb;
      return 0;
   }

   static DWORD CALLBACK MyStreamInCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
   {
      STREAMINFO *pInfo = (STREAMINFO *)dwCookie;
      if( cb > pInfo->length-pInfo->pos ) cb = pInfo->length-pInfo->pos;
      ::CopyMemory( pbBuff, pInfo->s.Mid(pInfo->pos, cb), cb);
      pInfo->pos += cb;
      *pcb = cb;
      return 0;
   }

   // -----------------------------------

   CComBSTR GetText()
   {
      CString txt;
      STREAMINFO info;
      info.s = txt;
      info.pos = 0;
      info.length = 0;
      EDITSTREAM es;
      es.dwCookie = (DWORD)&info;
      es.pfnCallback = MyStreamOutCallback;
      StreamOut(SF_TEXT, es);
      return CComBSTR(info.s);
   }
   
   void SetText(CComBSTR &bstr)
   {
      USES_CONVERSION;
      LPCTSTR pstr = OLE2CT(bstr);
      CString txt;
      CString s;
      s.LoadString(IDS_RTF_START);
      txt += s;
      s = CHtmlFormat::Format(pstr);
      txt += s;
      s.LoadString(IDS_RTF_END);
      txt += s;
      
      STREAMINFO info;
      info.s = txt;
      info.pos = 0;
      info.length = txt.GetLength();
      EDITSTREAM es;
      es.dwCookie = (DWORD)&info;
      es.pfnCallback = MyStreamInCallback;    
      m_bIsLoading = true;
      StreamIn(SF_RTF, es);
      m_bIsLoading = false;
      SetModify(FALSE);
   }
   
};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_SOURCEVIEW_H__5574832C_580A_4772_A047_5B5233984B67__INCLUDED_)
