// MainFrm.h : interface of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINFRM_H__0824DC2C_826C_11D4_8195_00C04F0F05BA__INCLUDED_)
#define AFX_MAINFRM_H__0824DC2C_826C_11D4_8195_00C04F0F05BA__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


class CMainFrame : 
   public CFrameWindowImpl<CMainFrame>, 
   public CUpdateUI<CMainFrame>,
   public CMessageFilter, 
   public CIdleHandler
{
public:
   DECLARE_FRAME_WND_CLASS(NULL, IDR_MAINFRAME)

   CDialogEdView m_view;
   CCommandBarCtrl m_CmdBar;

   BEGIN_MSG_MAP(CMainFrame)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      MESSAGE_HANDLER(WM_SIZE, OnSize)
      COMMAND_ID_HANDLER(ID_APP_EXIT, OnFileExit)
      COMMAND_ID_HANDLER(ID_VIEW_STATUS_BAR, OnViewStatusBar)
      COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAppAbout)
      NOTIFY_CODE_HANDLER(TCN_SELCHANGE, OnTabSelect)
      CHAIN_MSG_MAP(CUpdateUI<CMainFrame>)
      CHAIN_MSG_MAP(CFrameWindowImpl<CMainFrame>)
      CHAIN_MSG_MAP_MEMBER(m_view)
   END_MSG_MAP()

   BEGIN_UPDATE_UI_MAP(CMainFrame)
      UPDATE_ELEMENT(ID_VIEW_TOOLBAR, UPDUI_MENUPOPUP)
      UPDATE_ELEMENT(ID_VIEW_STATUS_BAR, UPDUI_MENUPOPUP)
      UPDATE_ELEMENT(ID_TOOL_ARROW, UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_TOOL_LABEL, UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_TOOL_EDIT, UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_TOOL_BUTTON, UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_TOOL_CHOICE, UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_TOOL_CHECKBOX, UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_TOOL_OPTION, UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_TOOL_LIST, UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_TOOL_FRAME, UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_TOOL_IMAGE, UPDUI_TOOLBAR)
   END_UPDATE_UI_MAP()

   // -----------------------------------

   virtual BOOL PreTranslateMessage(MSG* pMsg)
   {
      if(CFrameWindowImpl<CMainFrame>::PreTranslateMessage(pMsg)) return TRUE;
      return m_view.PreTranslateMessage(pMsg);
   }

   virtual BOOL OnIdle()
   {
      UIUpdateToolBar();
      return FALSE;
   }

   // -----------------------------------

   LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      // Create command bar window
      HWND hWndCmdBar = m_CmdBar.Create(m_hWnd, rcDefault, NULL, ATL_SIMPLE_CMDBAR_PANE_STYLE);
      // Attach menu
      m_CmdBar.AttachMenu(GetMenu());
      // Load command bar images
      m_CmdBar.LoadImages(IDR_MAINFRAME);
      // Remove old menu
      SetMenu(NULL);

      HWND hWndToolBar2 = CreateSimpleToolBarCtrl(m_hWnd, IDR_TOOLS, FALSE, ATL_SIMPLE_TOOLBAR_PANE_STYLE);
      HWND hWndToolBar1 = CreateSimpleToolBarCtrl(m_hWnd, IDR_MAINFRAME, FALSE, ATL_SIMPLE_TOOLBAR_PANE_STYLE);

      CreateSimpleReBar(ATL_SIMPLE_REBAR_NOBORDER_STYLE);
      AddSimpleReBarBand(hWndCmdBar);
      AddSimpleReBarBand(hWndToolBar1, NULL, TRUE);
      AddSimpleReBarBand(hWndToolBar2, NULL, TRUE);
      CreateSimpleStatusBar();

      m_hWndClient = m_view.Create(m_hWnd, 
         rcDefault, 
         NULL, 
         WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | TCS_TABS | TCS_BOTTOM | TCS_FOCUSNEVER, 
         0);

      UIAddToolBar(hWndToolBar1);
      UIAddToolBar(hWndToolBar2);
      UISetCheck(ID_VIEW_TOOLBAR, 1);
      UISetCheck(ID_VIEW_STATUS_BAR, 1);

      SendMessage(WM_COMMAND, ID_TOOL_ARROW, 0);

      CMessageLoop* pLoop = _Module.GetMessageLoop();
      pLoop->AddMessageFilter(this);
      pLoop->AddIdleHandler(this);

      return 0;
   }

   LRESULT OnTabSelect(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& bHandled)
   {
      for( int i=ID_TOOL_START; i<=ID_TOOL_END; i++ ) {
         UIEnable(i,m_view.GetCurSel()==0);
      }
      bHandled = FALSE; // let client get reflection
      return 0;
   }

   LRESULT OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
   {
      CFrameWindowImpl<CMainFrame>::OnSize(uMsg,wParam,lParam,bHandled);
      bHandled = TRUE;
      return 0;
   }

   LRESULT OnFileExit(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      PostMessage(WM_CLOSE);
      return 0;
   }

   LRESULT OnViewStatusBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      BOOL bNew = !::IsWindowVisible(m_hWndStatusBar);
      ::ShowWindow(m_hWndStatusBar, bNew ? SW_SHOWNOACTIVATE : SW_HIDE);
      UISetCheck(ID_VIEW_STATUS_BAR, bNew);
      UpdateLayout();
      return 0;
   }

   LRESULT OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CAboutDlg dlg;
      dlg.DoModal();
      return 0;
   }
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINFRM_H__0824DC2C_826C_11D4_8195_00C04F0F05BA__INCLUDED_)
